document.addEventListener('DOMContentLoaded', function () {
    if (typeof isAdmin === 'undefined' || !isAdmin) {
        console.warn("Grid controls are only available for admin users.");
        return;
    }

    // Utility functions
    function debounce(func, wait) {
        let timeout;
        return function executedFunction(...args) {
            const later = () => {
                clearTimeout(timeout);
                func(...args);
            };
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
        };
    }

    // Cache for grid calculations
    const gridCache = {
        dimensions: {
            columnCount: null,
            maxWidth: null,
            gutterWidth: null,
            rowHeight: null,
            rowCount: null,
            baselineHeight: null
        },
        lastUpdate: 0
    };

    // Grid state management
    const gridState = {
        isColumnVisible: true,
        isBaselineVisible: true,
        isRowVisible: true,
        scrollTimeout: null
    };

    // Create overlay and controls
    document.body.insertAdjacentHTML('beforeend', `
        <div class="grid-overlay" style="position: absolute; top: 0; left: 0; width: 100%; height: 100%; pointer-events: none; z-index: 1;">
            <div class="column-grid" id="columnGrid" style="position: absolute; top: 0; left: 50%; transform: translateX(-50%);"></div>
            <div class="baseline-grid" id="baselineGrid" style="position: absolute; top: 0; left: 0; width: 100%;"></div>
        </div>

        <div class="grid-controls" id="gridControls" style="position: fixed; bottom: 30px; right: 30px; width: 240px; background-color: rgb(40, 38, 41); padding: 10px; border-radius: 8px; z-index: 1000;">
            <div id="header" style="cursor: grab; background: rgba(0, 0, 0, 0.8); padding: 8px; text-align: center; color: white; border-radius: 8px 8px 0 0; display: flex; justify-content: space-between; align-items: center;">
                <span>Grid Controls</span>
                <div>
                    <button id="settingsButton" style="background: none; border: none; color: white; margin-right: 8px;">⚙️</button>
                    <button id="toggleInterface" style="background: none; border: none; color: white;">−</button>
                </div>
            </div>
            
            <!-- Toggle buttons -->
            <div style="display: flex; gap: 4px; padding: 8px; background: rgba(255, 255, 255, 0.1); margin: 8px 0; border-radius: 4px;">
                <button id="toggleColumns" style="flex: 1;">Toggle Columns</button>
                <button id="toggleBaseline" style="flex: 1;">Toggle Baseline</button>
                <button id="toggleRows" style="flex: 1;">Toggle Rows</button>
            </div>

            <div id="controlPanel">
                <!-- Settings Panel -->
                <div id="settingsPanel" style="display: none; background: rgba(0, 0, 0, 0.2); padding: 8px; border-radius: 4px;">
                    <h4 style="margin: 0 0 8px 0; color: white;">Visible Sections</h4>
                    <div style="display: flex; flex-direction: column; gap: 4px;">
                        ${['columnSettings', 'rowSettings', 'dimensionSettings', 'visualSettings'].map(section => `
                            <label style="color: white; display: flex; align-items: center;">
                                <input type="checkbox" checked data-section="${section}"> ${section.replace('Settings', '').charAt(0).toUpperCase() + section.slice(1).replace('Settings', '')}
                            </label>
                        `).join('')}
                    </div>
                </div>

                <!-- Column Settings Section -->
                <div class="section" id="columnSettingsSection">
                    <div class="section-header">
                        <span>Column Settings</span>
                        <span class="toggle-icon">+</span>
                    </div>
                    <div class="section-content collapsed">
                        <label>Column Count:</label>
                        <input id="columnCountInput" type="number" value="12" style="width: 100%;">
                    </div>
                </div>

                <!-- Row Settings Section -->
                <div class="section" id="rowSettingsSection">
                    <div class="section-header">
                        <span>Row Settings</span>
                        <span class="toggle-icon">+</span>
                    </div>
                    <div class="section-content collapsed">
                        <label>Row Height (px):</label>
                        <input id="rowHeightInput" type="number" value="100" style="width: 100%;">
                        <label>Row Spacing (px):</label>
                        <input id="rowSpacingInput" type="number" value="24" style="width: 100%;">
                        <label>Row Count:</label>
                        <input id="rowCountInput" type="number" value="0" style="width: 100%;">
                    </div>
                </div>

                <!-- Dimension Settings Section -->
                <div class="section" id="dimensionSettingsSection">
                    <div class="section-header">
                        <span>Dimensions</span>
                        <span class="toggle-icon">+</span>
                    </div>
                    <div class="section-content collapsed">
                        <label>Max Width (px):</label>
                        <input id="maxWidthInput" type="number" value="1290" style="width: 100%;">
                        <label>Gutter Width (px):</label>
                        <input id="gutterWidthInput" type="number" value="24" style="width: 100%;">
                        <label>Baseline Height (px):</label>
                        <input id="baselineHeightInput" type="number" value="8" style="width: 100%;">
                    </div>
                </div>

                <!-- Visual Settings Section -->
                <div class="section" id="visualSettingsSection">
                    <div class="section-header">
                        <span>Visual Settings</span>
                        <span class="toggle-icon">+</span>
                    </div>
                    <div class="section-content collapsed">
                        <label>Grid Line Colors:</label>
                        <div style="display: flex; gap: 4px;">
                            <input id="evenLineColor" type="color" value="#0066ff" style="width: 48%;">
                            <input id="oddLineColor" type="color" value="#3385ff" style="width: 48%;">
                        </div>
                        <label>Grid Opacity:</label>
                        <input id="opacitySlider" type="range" min="0" max="1" step="0.1" value="0.2" style="width: 100%;">
                    </div>
                </div>
            </div>
        </div>
    `);

    // Add styles
    const style = document.createElement('style');
    style.textContent = `
        .grid-line {
            position: absolute;
            left: 0;
            right: 0;
            height: 1px;
            pointer-events: none;
            will-change: transform;
        }
        .grid-column {
            height: 100%;
            transition: background-color 0.2s ease;
            will-change: transform, opacity;
        }
        .grid-row {
            position: absolute;
            left: 0;
            right: 0;
            will-change: transform;
        }
        .section {
            background: rgba(255, 255, 255, 0.1);
            padding: 8px;
            border-radius: 4px;
            margin-bottom: 4px;
        }
        .section-header {
            color: white;
            cursor: pointer;
            display: flex;
            justify-content: space-between;
            align-items: center;
        }
        .section-content {
            overflow: hidden;
            transition: max-height 0.3s ease;
            margin-top: 8px;
        }
        .section-content.collapsed {
            display: none;
        }
        .toggle-icon {
            transition: transform 0.3s ease;
        }
        #gridControls button {
            background: rgba(255, 255, 255, 0.1);
            border: none;
            color: white;
            padding: 4px 8px;
            border-radius: 4px;
            cursor: pointer;
        }
        #gridControls button:hover {
            background: rgba(255, 255, 255, 0.2);
        }
        #gridControls input, #gridControls select {
            background: rgba(255, 255, 255, 0.1);
            border: 1px solid rgba(255, 255, 255, 0.2);
            color: white;
            padding: 4px;
            border-radius: 4px;
            margin: 4px 0;
        }
        #gridControls label {
            color: rgba(255, 255, 255, 0.8);
            font-size: 0.9em;
            display: block;
            margin-top: 4px;
        }
    `;
    document.head.appendChild(style);

// === END OF PART 1 === //

// Get DOM elements
    const columnGrid = document.getElementById('columnGrid');
    const baselineGrid = document.getElementById('baselineGrid');
    const gridControls = document.getElementById('gridControls');
    const controlPanel = document.getElementById('controlPanel');
    const settingsButton = document.getElementById('settingsButton');
    const settingsPanel = document.getElementById('settingsPanel');
    const toggleInterfaceButton = document.getElementById('toggleInterface');

    // Grid Generation Functions
function generateColumnGrid() {
        const columnCount = parseInt(document.getElementById('columnCountInput').value);
        const maxWidth = parseInt(document.getElementById('maxWidthInput').value);
        const gutterWidth = parseInt(document.getElementById('gutterWidthInput').value);
        const opacity = document.getElementById('opacitySlider').value;
        const evenColor = document.getElementById('evenLineColor').value;
        const oddColor = document.getElementById('oddLineColor').value;

        // Check cache to avoid unnecessary updates
        if (gridCache.dimensions.columnCount === columnCount &&
            gridCache.dimensions.maxWidth === maxWidth &&
            gridCache.dimensions.gutterWidth === gutterWidth) {
            return;
        }

        // Update cache
        Object.assign(gridCache.dimensions, { columnCount, maxWidth, gutterWidth });

        // Calculate widths
        const totalGutterWidth = gutterWidth * (columnCount - 1);
        const availableWidth = maxWidth - totalGutterWidth;
        const columnWidth = availableWidth / columnCount;

        // Calculate the maximum height considering various elements
        const calculateMaxHeight = () => {
            const bodyHeight = Math.max(document.documentElement.scrollHeight, document.body.scrollHeight);
            const editorElement = document.querySelector('.monaco-editor');
            const editorHeight = editorElement ? editorElement.scrollHeight : 0;
            return Math.max(bodyHeight, editorHeight, window.innerHeight);
        };

        // Set container styles
        columnGrid.style.cssText = `
            position: fixed;
            top: 0;
            left: 50%;
            transform: translateX(-50%);
            width: ${maxWidth}px;
            display: grid;
            grid-template-columns: repeat(${columnCount}, ${columnWidth}px);
            column-gap: ${gutterWidth}px;
            height: ${calculateMaxHeight()}px;
            box-sizing: border-box;
        `;

        // Create document fragment for better performance
        const fragment = document.createDocumentFragment();
        
        // Generate columns
        for (let i = 0; i < columnCount; i++) {
            const column = document.createElement('div');
            column.className = 'grid-column';
            column.style.cssText = `
                background-color: ${i % 2 === 0 ? evenColor : oddColor};
                opacity: ${opacity};
                width: 100%;
                height: 100%;
            `;
            fragment.appendChild(column);
        }

        // Clear and append
        columnGrid.innerHTML = '';
        columnGrid.appendChild(fragment);
    }

    function generateRowGrid() {
        const rowHeight = parseInt(document.getElementById('rowHeightInput').value);
        const rowSpacing = parseInt(document.getElementById('rowSpacingInput').value);
        const rowCount = parseInt(document.getElementById('rowCountInput').value);
        const opacity = document.getElementById('opacitySlider').value;
        const evenColor = document.getElementById('evenLineColor').value;

        // Check cache
        if (gridCache.dimensions.rowHeight === rowHeight &&
            gridCache.dimensions.rowCount === rowCount) {
            return;
        }

        // Update cache
        Object.assign(gridCache.dimensions, { rowHeight, rowCount });

        // Remove existing row grid if any
        const existingRowGrid = document.querySelector('.row-grid');
        if (existingRowGrid) {
            existingRowGrid.remove();
        }

        const fragment = document.createDocumentFragment();
        const rowContainer = document.createElement('div');
        rowContainer.className = 'row-grid';
        rowContainer.style.cssText = `
            position: absolute;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            pointer-events: none;
        `;

        for (let i = 0; i < rowCount; i++) {
            const row = document.createElement('div');
            row.className = 'grid-row';
            row.style.cssText = `
                top: ${i * (rowHeight + rowSpacing)}px;
                height: ${rowHeight}px;
                background-color: ${evenColor};
                opacity: ${opacity};
            `;
            rowContainer.appendChild(row);
        }

        fragment.appendChild(rowContainer);
        document.querySelector('.grid-overlay').appendChild(fragment);
    }

    function generateBaselineGrid() {
        const baselineHeight = parseInt(document.getElementById('baselineHeightInput').value);
        const opacity = document.getElementById('opacitySlider').value;
        const evenColor = document.getElementById('evenLineColor').value;
        
        // Virtual scrolling implementation
        const viewportHeight = window.innerHeight;
        const scrollTop = window.scrollY;
        const buffer = viewportHeight * 0.5;
        
        const startRow = Math.max(0, Math.floor((scrollTop - buffer) / baselineHeight));
        const endRow = Math.ceil((scrollTop + viewportHeight + buffer) / baselineHeight);
        
        const fragment = document.createDocumentFragment();
        
        for (let i = startRow; i < endRow; i++) {
            const line = document.createElement('div');
            line.className = 'grid-line';
            line.style.cssText = `
                top: ${i * baselineHeight}px;
                background-color: ${evenColor};
                opacity: ${opacity};
            `;
            fragment.appendChild(line);
        }
        
        baselineGrid.innerHTML = '';
        baselineGrid.appendChild(fragment);
    }

    // Event Handlers
    // Section collapse functionality
    document.querySelectorAll('.section-header').forEach(header => {
        header.addEventListener('click', () => {
            const content = header.nextElementSibling;
            const icon = header.querySelector('.toggle-icon');
            content.classList.toggle('collapsed');
            icon.textContent = content.classList.contains('collapsed') ? '+' : '−';
        });
    });

    // Settings panel toggle
    settingsButton.addEventListener('click', () => {
        settingsPanel.style.display = settingsPanel.style.display === 'none' ? 'block' : 'none';
    });

    // Optimized update function
    const updateGrid = debounce(() => {
        requestAnimationFrame(() => {
            if (Date.now() - gridCache.lastUpdate < 16) return; // Skip if last update was less than 16ms ago
            
            if (gridState.isColumnVisible) generateColumnGrid();
            if (gridState.isRowVisible) generateRowGrid();
            if (gridState.isBaselineVisible) generateBaselineGrid();
            
            gridCache.lastUpdate = Date.now();
        });
    }, 16);

    // Toggle buttons
    document.getElementById('toggleColumns').addEventListener('click', () => {
        gridState.isColumnVisible = !gridState.isColumnVisible;
        columnGrid.style.display = gridState.isColumnVisible ? '' : 'none';
        // Also toggle rows when columns are toggled
        const rowGrid = document.querySelector('.row-grid');
        if (rowGrid) {
            rowGrid.style.display = gridState.isColumnVisible ? '' : 'none';
        }
    });

    document.getElementById('toggleBaseline').addEventListener('click', () => {
        gridState.isBaselineVisible = !gridState.isBaselineVisible;
        baselineGrid.style.display = gridState.isBaselineVisible ? '' : 'none';
    });

    document.getElementById('toggleRows').addEventListener('click', () => {
        gridState.isRowVisible = !gridState.isRowVisible;
        const rowGrid = document.querySelector('.row-grid');
        if (rowGrid) {
            rowGrid.style.display = gridState.isRowVisible ? '' : 'none';
        }
    });

    // Interface collapse/expand
    toggleInterfaceButton.addEventListener('click', () => {
        const isCollapsed = controlPanel.style.display === 'none';
        controlPanel.style.display = isCollapsed ? '' : 'none';
        toggleInterfaceButton.textContent = isCollapsed ? '−' : '+';
    });

    // Section visibility controls
    document.querySelectorAll('#settingsPanel input[type="checkbox"]').forEach(checkbox => {
        checkbox.addEventListener('change', () => {
            const sectionId = checkbox.dataset.section + 'Section';
            const section = document.getElementById(sectionId);
            if (section) {
                section.style.display = checkbox.checked ? 'block' : 'none';
            }
        });
    });

    // Input event handlers with debouncing
    document.querySelectorAll('input').forEach(input => {
        const debouncedUpdate = debounce(updateGrid, 16);
        input.addEventListener('change', debouncedUpdate);
        input.addEventListener('input', debouncedUpdate);
    });

    // Make controls draggable
    let isDragging = false;
    let currentX = 0, currentY = 0, initialX = 0, initialY = 0;

    document.getElementById('header').addEventListener('mousedown', e => {
        isDragging = true;
        initialX = e.clientX - currentX;
        initialY = e.clientY - currentY;
    });

    document.addEventListener('mousemove', e => {
        if (isDragging) {
            e.preventDefault();
            currentX = e.clientX - initialX;
            currentY = e.clientY - initialY;
            gridControls.style.transform = `translate(${currentX}px, ${currentY}px)`;
        }
    });

    document.addEventListener('mouseup', () => isDragging = false);

    // Optimized scroll handler
    const debouncedScrollUpdate = debounce(() => {
        if (gridState.isBaselineVisible) {
            generateBaselineGrid();
        }
    }, 16);

    window.addEventListener('scroll', debouncedScrollUpdate, { passive: true });

    // Optimized resize observer
    const resizeObserver = new ResizeObserver(debounce(() => {
        updateGrid();
    }, 100));
    resizeObserver.observe(document.body);

    // Initialize
    updateGrid();
});